@@ -237,8 +237,7 @@ static int add_xattr_item(struct btrfs_trans_handle *trans,
 	if (ret < 0) {
 		if (errno == ENOTSUP)
 			return 0;
-		error("getting a list of xattr failed for %s: %s", file_name,
-				strerror(errno));
+		error("getting a list of xattr failed for %s: %m", file_name);
 		return ret;
 	}
 	if (ret == 0)
@@ -253,8 +252,8 @@ static int add_xattr_item(struct btrfs_trans_handle *trans,
 		if (ret < 0) {
 			if (errno == ENOTSUP)
 				return 0;
-			error("getting a xattr value failed for %s attr %s: %s",
-				file_name, cur_name, strerror(errno));
+			error("getting a xattr value failed for %s attr %s: %m",
+				file_name, cur_name);
 			return ret;
 		}
 
@@ -262,8 +261,9 @@ static int add_xattr_item(struct btrfs_trans_handle *trans,
 					      cur_name_len, cur_value,
 					      ret, objectid);
 		if (ret) {
-			error("inserting a xattr item failed for %s: %s",
-					file_name, strerror(-ret));
+			errno = -ret;
+			error("inserting a xattr item failed for %s: %m",
+					file_name);
 		}
 
 		cur_name = strtok(next_location, &delimiter);
@@ -281,7 +281,7 @@ static int add_symbolic_link(struct btrfs_trans_handle *trans,
 
 	ret = readlink(path_name, buf, sizeof(buf));
 	if (ret <= 0) {
-		error("readlink failed for %s: %s", path_name, strerror(errno));
+		error("readlink failed for %s: %m", path_name);
 		goto fail;
 	}
 	if (ret >= sizeof(buf)) {
@@ -320,7 +320,7 @@ static int add_file_items(struct btrfs_trans_handle *trans,
 
 	fd = open(path_name, O_RDONLY);
 	if (fd == -1) {
-		error("cannot open %s: %s", path_name, strerror(errno));
+		error("cannot open %s: %m", path_name);
 		return ret;
 	}
 
@@ -339,10 +339,9 @@ static int add_file_items(struct btrfs_trans_handle *trans,
 
 		ret_read = pread64(fd, buffer, st->st_size, bytes_read);
 		if (ret_read == -1) {
-			error("cannot read %s at offset %llu length %llu: %s",
+			error("cannot read %s at offset %llu length %llu: %m",
 				path_name, (unsigned long long)bytes_read,
-				(unsigned long long)st->st_size,
-				strerror(errno));
+				(unsigned long long)st->st_size);
 			free(buffer);
 			goto end;
 		}
@@ -388,11 +387,10 @@ again:
 		ret_read = pread64(fd, eb->data, sectorsize, file_pos +
 				   bytes_read);
 		if (ret_read == -1) {
-			error("cannot read %s at offset %llu length %llu: %s",
+			error("cannot read %s at offset %llu length %llu: %m",
 				path_name,
 				(unsigned long long)file_pos + bytes_read,
-				(unsigned long long)sectorsize,
-				strerror(errno));
+				(unsigned long long)sectorsize);
 			goto end;
 		}
 
@@ -467,7 +465,7 @@ static int traverse_directory(struct btrfs_trans_handle *trans,
 	dir_entry->dir_name = dir_name;
 	dir_entry->path = realpath(dir_name, NULL);
 	if (!dir_entry->path) {
-		error("realpath failed for %s: %s", dir_name, strerror(errno));
+		error("realpath failed for %s: %m", dir_name);
 		ret = -1;
 		goto fail_no_dir;
 	}
@@ -506,8 +504,8 @@ static int traverse_directory(struct btrfs_trans_handle *trans,
 		parent_inum = parent_dir_entry->inum;
 		parent_dir_name = parent_dir_entry->dir_name;
 		if (chdir(parent_dir_entry->path)) {
-			error("chdir failed for %s: %s",
-				parent_dir_name, strerror(errno));
+			error("chdir failed for %s: %m",
+				parent_dir_name);
 			ret = -1;
 			goto fail_no_files;
 		}
@@ -515,8 +513,8 @@ static int traverse_directory(struct btrfs_trans_handle *trans,
 		count = scandir(parent_dir_entry->path, &files,
 				directory_select, NULL);
 		if (count == -1) {
-			error("scandir failed for %s: %s",
-				parent_dir_name, strerror(errno));
+			error("scandir failed for %s: %m",
+				parent_dir_name);
 			ret = -1;
 			goto fail;
 		}
@@ -525,8 +523,8 @@ static int traverse_directory(struct btrfs_trans_handle *trans,
 			cur_file = files[i];
 
 			if (lstat(cur_file->d_name, &st) == -1) {
-				error("lstat failed for %s: %s",
-					cur_file->d_name, strerror(errno));
+				error("lstat failed for %s: %m",
+					cur_file->d_name);
 				ret = -1;
 				goto fail;
 			}
@@ -648,7 +646,7 @@ int btrfs_mkfs_fill_dir(const char *source_dir, struct btrfs_root *root,
 
 	ret = lstat(source_dir, &root_st);
 	if (ret) {
-		error("unable to lstat %s: %s", source_dir, strerror(errno));
+		error("unable to lstat %s: %m", source_dir);
 		ret = -errno;
 		goto out;
 	}
@@ -726,8 +724,8 @@ u64 btrfs_mkfs_size_dir(const char *dir_name, u32 sectorsize, u64 min_dev_size,
 	u64 meta_threshold = SZ_8M;
 	u64 data_threshold = SZ_8M;
 
-	float data_multipler = 1;
-	float meta_multipler = 1;
+	float data_multiplier = 1;
+	float meta_multiplier = 1;
 
 	fs_block_size = sectorsize;
 	ftw_data_size = 0;
@@ -739,8 +737,7 @@ u64 btrfs_mkfs_size_dir(const char *dir_name, u32 sectorsize, u64 min_dev_size,
 	 */
 	ret = nftw(dir_name, ftw_add_entry_size, 10, FTW_PHYS);
 	if (ret < 0) {
-		error("ftw subdir walk of %s failed: %s", dir_name,
-			strerror(errno));
+		error("ftw subdir walk of %s failed: %m", dir_name);
 		exit(1);
 	}
 
@@ -766,11 +763,11 @@ u64 btrfs_mkfs_size_dir(const char *dir_name, u32 sectorsize, u64 min_dev_size,
 	/* Minimal chunk size from btrfs_alloc_chunk(). */
 	if (meta_profile & BTRFS_BLOCK_GROUP_DUP) {
 		meta_threshold = SZ_32M;
-		meta_multipler = 2;
+		meta_multiplier = 2;
 	}
 	if (data_profile & BTRFS_BLOCK_GROUP_DUP) {
 		data_threshold = SZ_64M;
-		data_multipler = 2;
+		data_multiplier = 2;
 	}
 
 	/*
@@ -780,10 +777,10 @@ u64 btrfs_mkfs_size_dir(const char *dir_name, u32 sectorsize, u64 min_dev_size,
 	 */
 	if (meta_size > meta_threshold)
 		meta_chunk_size = (round_up(meta_size, meta_threshold) -
-				   meta_threshold) * meta_multipler;
+				   meta_threshold) * meta_multiplier;
 	if (ftw_data_size > data_threshold)
 		data_chunk_size = (round_up(ftw_data_size, data_threshold) -
-				   data_threshold) * data_multipler;
+				   data_threshold) * data_multiplier;
 
 	total_size = data_chunk_size + meta_chunk_size + min_dev_size;
 	return total_size;
@@ -862,8 +859,8 @@ static int set_device_size(struct btrfs_fs_info *fs_info,
 	trans = btrfs_start_transaction(chunk_root, 1);
 	if (IS_ERR(trans)) {
 		ret = PTR_ERR(trans);
-		error("failed to start transaction: %d (%s)", ret,
-			strerror(-ret));
+		errno = -ret;
+		error("failed to start transaction: %d (%m)", ret);
 		return ret;
 	}
 	key.objectid = BTRFS_DEV_ITEMS_OBJECTID;
@@ -891,9 +888,10 @@ static int set_device_size(struct btrfs_fs_info *fs_info,
 	 * super->dev_item
 	 */
 	ret = btrfs_commit_transaction(trans, chunk_root);
-	if (ret < 0)
-		error("failed to commit current transaction: %d (%s)",
-			ret, strerror(-ret));
+	if (ret < 0) {
+		errno = -ret;
+		error("failed to commit current transaction: %d (%m)", ret);
+	}
 	btrfs_release_path(&path);
 	return ret;
 
@@ -928,8 +926,8 @@ int btrfs_mkfs_shrink_fs(struct btrfs_fs_info *fs_info, u64 *new_size_ret,
 
 	ret = get_device_extent_end(fs_info, 1, &new_size);
 	if (ret < 0) {
-		error("failed to get minimal device size: %d (%s)",
-			ret, strerror(-ret));
+		errno = -ret;
+		error("failed to get minimal device size: %d (%m)", ret);
 		return ret;
 	}
 
@@ -946,16 +944,15 @@ int btrfs_mkfs_shrink_fs(struct btrfs_fs_info *fs_info, u64 *new_size_ret,
 	if (shrink_file_size) {
 		ret = fstat64(device->fd, &file_stat);
 		if (ret < 0) {
-			error("failed to stat devid %llu: %s", device->devid,
-				strerror(errno));
+			error("failed to stat devid %llu: %m", device->devid);
 			return ret;
 		}
 		if (!S_ISREG(file_stat.st_mode))
 			return ret;
 		ret = ftruncate64(device->fd, new_size);
 		if (ret < 0) {
-			error("failed to truncate device file of devid %llu: %s",
-				device->devid, strerror(errno));
+			error("failed to truncate device file of devid %llu: %m",
+				device->devid);
 			return ret;
 		}
 	}
