@@ -3,7 +3,7 @@
 # iterate over all fuzzed images and run 'btrfs check', try various options to
 # get more code coverage
 
-source $TOP/tests/common
+source "$TEST_TOP/common"
 
 setup_root_helper
 check_prereq btrfs
@@ -21,6 +21,6 @@ check_image() {
 	run_mayfail $TOP/btrfs check --repair "$image"
 }
 
-check_all_images $TOP/tests/fuzz-tests/images
+check_all_images "$TEST_TOP/fuzz-tests/images"
 
 exit 0
